/*
 * Defines a Camera node.
 * Inherits from Transformable.
 */

#ifndef __RC_CAMERA_H__
#define __RC_CAMERA_H__

class Camera : public Transformable {
	friend class SceneGraph;
	friend class World;

/*----------------------------------------------------------[ Functions ]----*/

	protected:
		Camera(
				const char	*nname
			);
		~Camera();

	public:

		void setOrthogonalProjection(
				f32		left,
				f32		right,
				f32		bottom,
				f32		top,
				f32		nnear,
				f32		nfar
			);

		void setPerspectiveProjection(
				f32		fovy,			/* Field of view in radians */
				f32		aspect,			/* y / x */
				f32		nnear,
				f32		nfar
			);

		void setProjection(
				mat4f	&matrix
			);

	private:
		/*
		 * Recursively calculates the view transform, starting from
		 * the camera node and ends with the world node.
		 */
		void calculateViewTransform(
				mat4f	*matrix
			);

		Node *duplicateSelf();

/*------------------------------------------------------------[ Members ]----*/

	private:
		f32					nearPlane;
		f32					farPlane;
		mat4f				projectionMatrix;
		ProjectionType_t	projectionType;		/* Knowing the layout of the
												 * projection matrix can help
												 * optimization later on.
												 */

};


#endif /* __RC_CAMERA_H__ */

