#include "RenderChimp.h"
#include "SceneGraph.h"
#include "Resource.h"
#include "ResourceList.h"
#include "CubeMap.h"

/* Warnings */
static const Result_t WRN_CLONE = {
	RESULT_WARNING, "Cloning a cube map resource is not permitted" };

static const Result_t WRN_BPP = {
	RESULT_WARNING, "The texture doesn't have 1, 3 or 4 bytes per pixel" };

/*---------------------------------------------------------------------------*/

static void setCubeTex(
		const char				*filename,
		u32					side
	)
{
	u8 *pixels;
	u32 w, h, bpp;

	apeGetTexture(filename, &pixels, &w, &h, &bpp);

	if (bpp == 4) {
		glTexImage2D(side, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
	} else if (bpp == 3) {
		glTexImage2D(side, 0, GL_RGB, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, pixels);
	} else if (bpp == 1) {
		glTexImage2D(side, 0, GL_LUMINANCE, w, h, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, pixels);
	} else {
		REPORT(WRN_BPP);
		return;
	}

	GL_ASSERT();

	apeDestroyTexture(pixels);

}

/*---------------------------------------------------------------------------*/

CubeMap::CubeMap(
		const char				*nidentifier,

		const char				*nleft,
		const char				*nright,
		const char				*nup,
		const char				*ndown,
		const char				*nfront,
		const char				*nback,

		u32					purge_level

	) : Resource(nidentifier, purge_level, RESOURCE_CUBEMAP)
{
	glGenTextures(1, &texId); 
	glBindTexture(GL_TEXTURE_CUBE_MAP, texId);

	GL_ASSERT();

	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE); 
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

	GL_ASSERT();

	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_LINEAR); 

	GL_ASSERT();

	setCubeTex(nleft, GL_TEXTURE_CUBE_MAP_POSITIVE_X);
	setCubeTex(nright, GL_TEXTURE_CUBE_MAP_NEGATIVE_X);
	setCubeTex(nup, GL_TEXTURE_CUBE_MAP_POSITIVE_Y);
	setCubeTex(ndown, GL_TEXTURE_CUBE_MAP_NEGATIVE_Y);
	setCubeTex(nfront, GL_TEXTURE_CUBE_MAP_NEGATIVE_Z);
	setCubeTex(nback, GL_TEXTURE_CUBE_MAP_POSITIVE_Z);

	GL_ASSERT();
}

/*---------------------------------------------------------------------------*/

CubeMap::~CubeMap()
{
	glDeleteTextures(1, &texId);

	GL_ASSERT();
}

/*---------------------------------------------------------------------------*/

Resource *CubeMap::clone(
		const char		*clone_name,

		u32				purge_level
	)
{
	REPORT(WRN_CLONE);

	return 0;
}

/*---------------------------------------------------------------------------*/

void CubeMap::setCubeMap(
		CubeMap				**target_map,
		CubeMap				*new_map,
		bool				instantiate_resource
	)
{
	if (*target_map)
		sceneGraph.dereferenceResource(*target_map);

	if (instantiate_resource) {
		*target_map = (CubeMap *) new_map->instantiate();
	} else {
		*target_map = new_map;
	}
}

/*---------------------------------------------------------------------------*/

#ifdef DEBUG

void CubeMap::print(
		FILE		*f
	)
{

#ifdef LOG_TO_FILE
	#define prnt0(a)		fprintf(f, a)
	#define prnt1(a, b)		fprintf(f, a, b)
#else /* LOG_TO_FILE */
	#define prnt0(a)		printf(a)
	#define prnt1(a, b)		printf(a, b)
#endif /* LOG_TO_FILE */

	prnt1("    [ TEX ID   ][ %u ]\n", texId);

}

#endif /* DEBUG */

/*---------------------------------------------------------------------------*/

