/*
 * CubeMap.
 * Inherits from Resource.
 *
 * This is a cube environment map that can be used for nice reflections in your
 * shaders. It uses 6 faces organized in a cube, each with its own texture.
 */

#ifndef __RC_CUBEMAP_H__
#define __RC_CUBEMAP_H__

class CubeMap : public Resource {
	friend class SceneGraph;
	friend class Texture;

/*----------------------------------------------------------[ Functions ]----*/

	private:
		CubeMap(
				const char				*nidentifier,

				/* Texture filenames */
				const char				*nleft,
				const char				*nright,
				const char				*nup,
				const char				*ndown,
				const char				*nfront,
				const char				*nback,

				u32					purge_level
			);

		~CubeMap();

	public:
		Resource *clone(
				const char			*clone_name,

				u32					purge_level
			);

		GLuint getGLTexId() { return texId; }

	/* Static helper functions */

		static void setCubeMap(
				CubeMap				**target_map,
				CubeMap				*new_map,
				bool				instantiate_resource
			);

#ifdef DEBUG
	protected:
		void print(
				FILE	*f
			);
#endif /* DEBUG */

/*------------------------------------------------------------[ Members ]----*/

	private:
		GLuint				texId;

};

#endif /* __RC_CUBEMAP_H__ */

