#include "RenderChimp.h"
#include "SceneGraph.h"
#include "Light.h"

/*---------------------------------------------------------------------------*/

Light::Light(
		const char	*nname
	) : Transformable(nname, NODE_LIGHT)
{
	color = rgbf(1.0f, 1.0f, 1.0f);
	intensity = 1.0f;
}

/*---------------------------------------------------------------------------*/

Light::Light(
		const char	*nname,
		rgbf	ncolor,
		f32		nintensity
	) : Transformable(nname, NODE_LIGHT)

{
	color = ncolor;
	intensity = nintensity;
}

/*---------------------------------------------------------------------------*/

Light::~Light()
{
	if (parent) {
		Node *world_node = parent;

		while (world_node->getParent()) {
			world_node = world_node->getParent();
		}

		world_node->dereferenceLight(this);
	}
}

/*---------------------------------------------------------------------------*/

void Light::setColor(
		rgbf	ncolor
	)
{
	color = ncolor;
}

/*---------------------------------------------------------------------------*/

void Light::setIntensity(
		f32		nintensity
	)
{
	intensity = nintensity;
}

/*---------------------------------------------------------------------------*/

void Light::setFalloff(
		f32		nfalloff
	)
{
	falloff = nfalloff;
}

/*---------------------------------------------------------------------------*/

rgbf Light::getColor()
{
	return color;
}

/*---------------------------------------------------------------------------*/

f32 Light::getIntensity()
{
	return intensity;
}

/*---------------------------------------------------------------------------*/

f32 Light::getFalloff()
{
	return falloff;
}

/*---------------------------------------------------------------------------*/

Node *Light::duplicateSelf()
{
	Light *c = sceneGraph.createLight(name, color, intensity);

	return c;
}

/*---------------------------------------------------------------------------*/

