#include "RenderChimp.h"
#include "SceneGraph.h"
#include "Resource.h"
#include "ResourceList.h"
#include "Material.h"

#include "MaterialDirectional.h"
#include "MaterialPoint.h"
#include "MaterialColorful.h"
#include "MaterialWeird.h"
#include "MaterialTextured.h"
#include "MaterialReflective.h"


/* Warnings */
static const Result_t WRN_TEX_SLOT = {
	RESULT_WARNING, "Texture slot exceeds the allowed texture count." };


/*---------------------------------------------------------------------------*/

Material *Material::create(
		MaterialType_t			material_type,
		const char				*nidentifier,
		u32						purge_level
	)
{
	Material *a = 0;

	GL_ASSERT();

	switch (material_type) {
		default:
		case MATERIAL_DIRECTIONAL:
			a = new MaterialDirectional(nidentifier, purge_level);
			break;
		case MATERIAL_POINT:
			a = new MaterialPoint(nidentifier, purge_level);
			break;
		case MATERIAL_COLORFUL:
			a = new MaterialColorful(nidentifier, purge_level);
			break;
		case MATERIAL_WEIRD:
			a = new MaterialWeird(nidentifier, purge_level);
			break;
		case MATERIAL_TEXTURED:
			a = new MaterialTextured(nidentifier, purge_level);
			break;
		case MATERIAL_REFLECTIVE:
			a = new MaterialReflective(nidentifier, purge_level);
			break;
	}

	return a;
}


/*---------------------------------------------------------------------------*/

Material::Material(
		const char				*nidentifier,
		u32						purge_level,
		MaterialType_t			ntype
	) : Resource(nidentifier, purge_level, RESOURCE_MATERIAL)
{
	GL_ASSERT();

	materialType = ntype;

	shaderProgram = 0;
	size = 0;
}

/*---------------------------------------------------------------------------*/

Material::~Material() {

	ensure_heap();

	if (shaderProgram)
		sceneGraph.dereferenceResource(shaderProgram);

	ensure_heap();

}

/*---------------------------------------------------------------------------*/

void Material::prepareDrawing(
		FrameInfo_t			*di,
		Geometry			*geometry
	)
{
	if (di->activeShader != shaderProgram) {
		shaderProgram->activate();
		di->activeShader = shaderProgram;
	}

	prepareMaterial(di, geometry);
}

/*---------------------------------------------------------------------------*/

MaterialType_t Material::getMaterialType()
{
	return materialType;
}

/*---------------------------------------------------------------------------*/

#ifdef DEBUG

void Material::print(
		FILE		*f
	)
{
	char *c = shaderProgram->getIdentityStringCopy();

#ifdef LOG_TO_FILE
	#define prnt0(a)		fprintf(f, a)
	#define prnt1(a, b)		fprintf(f, a, b)
#else /* LOG_TO_FILE */
	#define prnt0(a)		printf(a)
	#define prnt1(a, b)		printf(a, b)
#endif /* LOG_TO_FILE */

	prnt1("    [ TYPE     ][ %u ]\n", (u32) materialType);
	prnt1("    [ S.PROG   ][ %s ]\n", c);

	free(c);
}

#endif /* DEBUG */

/*---------------------------------------------------------------------------*/

