/*
 * Material.
 * Inherits from Resource.
 *
 * Note: Don't forget to add the includes of new materials to the bottom of
 * this file!
 */

#ifndef __RC_MATERIAL_H__
#define __RC_MATERIAL_H__

/*--------------------------------------------------[ Material settings ]----*/

typedef enum {

	MATERIAL_DIRECTIONAL = 0,
	MATERIAL_POINT,
	MATERIAL_COLORFUL,
	MATERIAL_WEIRD,
	MATERIAL_TEXTURED,
	MATERIAL_REFLECTIVE,

	MATERIAL_TYPE_LAST

} MaterialType_t;

/*---------------------------------------------------------------------------*/

class Material : public Resource {
	friend class SceneGraph;
	friend class Geometry;
	friend class TriangleMesh;

/*----------------------------------------------------------[ Functions ]----*/

	protected:
		Material(
				const char			*nidentifier,
				u32					purge_level,
				MaterialType_t		ntype
			);

		virtual ~Material();

		void prepareDrawing(
				FrameInfo_t			*di,
				Geometry			*geometry
			);

		virtual void prepareMaterial(
				FrameInfo_t			*di,
				Geometry			*geometry
			) { };

	public:

		MaterialType_t getMaterialType();

	protected:

		static Material *create(
			MaterialType_t			material_type,
			const char				*nidentifier,
			u32						purge_level
		);

#ifdef DEBUG
	protected:
		void print(
				FILE	*f
			);
#endif /* DEBUG */

/*------------------------------------------------------------[ Members ]----*/

	protected:
		MaterialType_t				materialType;
		ShaderProgram				*shaderProgram;

};

#include "MaterialDirectional.h"
#include "MaterialPoint.h"
#include "MaterialColorful.h"
#include "MaterialWeird.h"
#include "MaterialTextured.h"
#include "MaterialReflective.h"


#endif /* __RC_MATERIAL_H__ */

