#if 1

/*
 * Memory debugger
 */

/* DON'T touch unless you know exactly what you're doing! */

#ifndef __RC_MEMORY_H__
#define __RC_MEMORY_H__

/* Memory declarations */

#ifndef MEMORY_DEBUG

		/* Release declarations */
	void *MemoryAllocate(
			const u32	mem_size
		);

	void *MemoryAllocateSetZero(
			const u32	n,
			const u32	type_size
		);

	void MemoryFree(
			void		*ptr
		);

	void *MemoryReallocate(
			void		*old_ptr,
			const u32	new_size
		);

#endif /* !MEMORY_DEBUG */

	/* Release/Debug declarations */
void *operator new(
		size_t		mem_size
	);

void *operator new [](
		size_t		mem_size
	);

void operator delete(
		void *ptr
	);

void operator delete [](
		void *ptr
	);

#ifdef MEMORY_DEBUG

		/* Debug declarations */
	void *MemoryAllocateDebug(
			u32			mem_size,
			const char	*file,
			const char	*func,
			int			line
		);

	void *MemoryAllocateSetZeroDebug(
			u32			n,
			u32			type_size,
			const char	*file,
			const char	*func,
			int			line
		);

	void MemoryFreeDebug(
			void		*ptr
		);

	void *MemoryReallocateDebug(
			void		*old_ptr,
			u32			new_size,
			const char	*file,
			const char	*func,
			int			line
		);

	i32 MemorySetInfo(
			const char	*new_file,
			const char	*new_func,
			i32			new_line
		);

#endif /* MEMORY_DEBUG */

#ifdef malloc
	#undef malloc
#endif

#ifdef calloc
	#undef calloc
#endif

#ifdef free
	#undef free
#endif

#ifdef realloc
	#undef realloc
#endif

#ifdef new
	#undef new
#endif

#ifdef delete
	#undef delete
#endif


#ifdef MEMORY_DEBUG

	#define malloc(mem_size)							MemoryAllocateDebug(mem_size, __FILE__, __FUNCTION__, __LINE__)
	#define calloc(n, type_size)						MemoryAllocateSetZeroDebug(n, type_size, __FILE__, __FUNCTION__, __LINE__)
	#define free(ptr)									MemoryFreeDebug(ptr)
	#define realloc(old_ptr, new_size)					MemoryReallocateDebug(old_ptr, new_size, __FILE__, __FUNCTION__, __LINE__)

	#define new											(MemorySetInfo(__FILE__, __FUNCTION__, __LINE__)) ? 0 : new 

#else /* MEMORY_DEBUG */

	#define malloc(mem_size)							MemoryAllocate(mem_size)
	#define calloc(n, mem_size)							MemoryAllocateSetZero(n, type_size)
	#define free(ptr)									MemoryFree(ptr)
	#define realloc(old_ptr, new_size)					MemoryReallocate(old_ptr, new_size)

#endif /* MEMORY_DEBUG */




u32 getAllocationSize();

u32 getAllocationPeak();

u32 getAllocationPeakLocal();

u32 getAllocationCount();

void logAllocations();

void ensure_heap();

#endif /* __RC_MEMORY_H__ */

#else

	#ifndef __RC_MEMORY_H__
	#define __RC_MEMORY_H__

		u32 getAllocationSize();

		u32 getAllocationPeak();

		u32 getAllocationPeakLocal();

		u32 getAllocationCount();

		void logAllocations();

		void ensure_heap();

	#endif

#endif