#include <cstring>
#include <cstdio>
#include "RenderChimp.h"
#include "Resource.h"
#include "Misc.h"

/*---------------------------------------------------------------------------*/

Resource::Resource(
		const char				*nidentifier,
		u32						purge_level,
		ResourceType_t			rtype
	)
{
	next = 0;
	identifier = duplicateString(nidentifier);
	hash = calculateStringHash(identifier);

	purgeLevel = purge_level;

	type = rtype;
}

/*---------------------------------------------------------------------------*/

Resource::~Resource() {

	if (identifier)
		free(identifier);

	GL_ASSERT();
}

/*---------------------------------------------------------------------------*/

bool Resource::isEqual(
		Resource		*res
	)
{
	return hash == res->hash &&
		strcmp(identifier, res->identifier) == 0;
}

/*---------------------------------------------------------------------------*/

Resource *Resource::instantiate()
{
	n_refs++;

	return this;
}

/*---------------------------------------------------------------------------*/

ResourceType_t Resource::getType()
{
	return type;
}

/*---------------------------------------------------------------------------*/

void Resource::setPurgeLevel(
		u32				purge_level
	)
{
	purgeLevel = purge_level;
}

/*---------------------------------------------------------------------------*/

void Resource::setMaxPurgeLevel(
		u32				purge_level
	)
{
	purgeLevel = purge_level > purgeLevel ? purge_level : purgeLevel;
}

/*---------------------------------------------------------------------------*/

u32 Resource::getPurgeLevel()
{
	return purgeLevel;
}

/*---------------------------------------------------------------------------*/

u32 Resource::getReferenceCount()
{
	return n_refs;
}

/*---------------------------------------------------------------------------*/

char *Resource::getIdentityStringCopy()
{
	return duplicateString(identifier);
}

/*---------------------------------------------------------------------------*/

