/*
 * Resource base class
 * 
 */

#ifndef __RC_RESOURCE_H__
#define __RC_RESOURCE_H__

/* Resource type */
typedef enum {
	RESOURCE_VERTEX_ARRAY = 0,
	RESOURCE_INDEX_ARRAY,
	RESOURCE_TEXTURE,
	RESOURCE_CUBEMAP,
	RESOURCE_SHADER,
	RESOURCE_SHADER_PROGRAM,
	RESOURCE_MATERIAL,

	RESOURCE_TYPE_LAST

} ResourceType_t;

class Resource {
	friend class SceneGraph;
	friend class ResourceList;

/*----------------------------------------------------------[ Functions ]----*/

	protected:
		Resource(
				const char				*nidentifier,
				u32						purge_level,
				ResourceType_t			rtype
			);

		virtual ~Resource();

	public:
		/*
		 * Compares the hash values and identifiers of this and the incoming
		 * resource. Return true if they are equal, otherwise false.
		 */
		virtual bool isEqual(
				Resource		*res
			);

		/* Creat a clone of a resource. This duplicates the data */
		/* Note: not all resources are clonable */
		virtual Resource *clone(
				const char		*clone_name,

				u32				purge_level
			) { return 0; }

		/* Instantiate a resource. This increases the reference counter */
		virtual Resource *instantiate();

		ResourceType_t getType();

		void setPurgeLevel(
				u32				purge_level
			);

		void setMaxPurgeLevel(
				u32				purge_level
			);

		u32 getPurgeLevel();

		u32 getReferenceCount();

		/* Returns a copy of the identifier, which you'll have to delete your-
		 * self when you're done with it
		 */
		char *getIdentityStringCopy();

	protected:
#ifdef DEBUG
		virtual void print(
				FILE	*f
			) { }
#endif /* DEBUG */

/*------------------------------------------------------------[ Members ]----*/

	private:
		u32				hash;			/* Hash value based on the
										 * "identifier".
										 */
		u32				n_refs;			/* Number of references to this
										 * resource object.
										 */
		Resource		*next;			/* Used for List hash table */

	protected:
		ResourceType_t	type;
		char			*identifier;

		u32				size;			/* Memory occupied by this resource */

		u32				purgeLevel;		/* Don't delete, even if there are no
										 * references to this resource
										 */

};

#endif /* __RC_RESOURCE_H__ */

