/*
 * Error and warning handling system
 */

#ifndef __RC_RESULT_H__
#define __RC_RESULT_H__

extern FILE *logfile;

/* Result error level */
typedef enum {
	RESULT_SUCCESS			= 0x0000,
	RESULT_WARNING,
	RESULT_ERROR
} ResultType_t;

/* Result info */
typedef struct {
	ResultType_t			type;
	const char					*message;
} Result_t;


/* Standard success */
static const Result_t OK = {
	RESULT_SUCCESS, "Everything is fine" };

/* Standard warning */
static const Result_t WRN = {
	RESULT_WARNING, "Undefined warning" };

/* Standard error */
static const Result_t ERR = {
	RESULT_ERROR, "Undefined error" };


/* Errors */
static const Result_t ERR_ARG = {
	RESULT_ERROR, "Invalid argument"};

static const Result_t ERR_GL = {
	RESULT_ERROR, "OpenGL doesn't like you" };

static const Result_t ERR_OUT_OF_MEM = {
	RESULT_ERROR, "Out of memory" };


#define SUCCESS(r)						((r).type == RESULT_SUCCESS)

#define FAIL(r)							(!SUCCESS(r))


/* Initiate the result file (if LOG_TO_FILE is defined) */
void ResultInit();

/* Wraps up the result log file */
void ResultRelease();

/* Report a result to a log file and standard output */
void report(
		Result_t			r,
		const char			*file,
		const char			*function,
		int					line
	);

/* Print message to screen, if possible */
void ResultUpdateMessage(
		u32					width
	);

/* Ensures that GL hasn't generated any errors */
void glAssert(
		const char			*file,
		const char			*function,
		int					line
	);

#ifdef DEBUG

	#define REPORT(r)					report((r), __FILE__, __FUNCTION__, __LINE__)
	
	#define GL_ASSERT()					glAssert(__FILE__, __FUNCTION__, __LINE__)

#else /* DEBUG */

	#define REPORT(r)					report((r), "?", "?", 0)

	#define GL_ASSERT()

#endif /* DEBUG */

#endif /* __RC_RESULT_H__ */
