#include <cstring>
#include <cstdio>
#include "SceneGraph.h"
#include "Shader.h"

#define SHADER_ERROR_BUFFER_LENGTH			8192

/* Warnings */
static const Result_t WRN_SOURCE = {
	RESULT_WARNING, "Couldn't load shader source" };

static const Result_t WRN_COMPILE = {
	RESULT_WARNING, "Couldn't compile shader" };

static const Result_t WRN_CLONE = {
	RESULT_WARNING, "Cloning a shader resource is not permitted" };

/*---------------------------------------------------------------------------*/
	
char *createPrefixString(
		char				**defs,
		u32					n_defs
	)
{
	u32 i;
	char str[4096];
	char *s;

	str[0] = '\0';

	for (i = 0; i < n_defs; i++) {
		sprintf(str, "%s#define %s\n", str, defs[i]);
	}

	s = duplicateString(str);

	return s;
}
	
/*---------------------------------------------------------------------------*/

Shader::Shader(
		const char			*new_key,
		char				*prefix,
		const char			*filename,
		ShaderType_t		type,
		u32					purge_level
	) : Resource(new_key, purge_level, RESOURCE_SHADER)
{
	GLsizei length;
	char *source[2];
	GLint status;
	FILE *file;

	/* Read source file */
	file = fopen(filename, "rb");

	if (!file) {
		REPORT(WRN_SOURCE);
		return;
	}

	fseek(file, 0, SEEK_END);
	length = ftell(file);
	fseek(file, 0, SEEK_SET);

	source[0] = prefix;

	source[1] = (char *) malloc(sizeof(char) * (length + 1));
	source[1][length] = '\0';

	if (!fread(source[1], length, 1, file)) {
		REPORT(WRN_SOURCE);
		return;
	}

	size = strlen(source[1]) + strlen(source[0]);

	fclose(file);

	/* Compile shader */

	switch (type) {
		case SHADER_TYPE_VERTEX:
			id = glCreateShader(GL_VERTEX_SHADER);
			break;
		default:
		case SHADER_TYPE_FRAGMENT:
			id = glCreateShader(GL_FRAGMENT_SHADER);
			break;
	}

	glShaderSource(id, 2, (const char **) source, NULL);

	glCompileShader(id);

	glGetShaderiv(id, GL_COMPILE_STATUS, &status);

	if (status == GL_FALSE) {
		char result[SHADER_ERROR_BUFFER_LENGTH];

		REPORT(WRN_COMPILE);

		glGetShaderInfoLog(id, SHADER_ERROR_BUFFER_LENGTH, &length, result);

		if (length) {
			Result_t msg = {RESULT_WARNING, result};

			REPORT(msg);
		}
	}

	free(source[1]);
}

/*---------------------------------------------------------------------------*/

Shader::~Shader()
{
	glDeleteShader(id);

	GL_ASSERT();
}

/*---------------------------------------------------------------------------*/

Resource *Shader::clone(
		const char			*clone_name,

		u32				purge_level
	)
{
	REPORT(WRN_CLONE);

	return 0;
}

/*---------------------------------------------------------------------------*/

#ifdef DEBUG

void Shader::print(
		FILE		*f
	)
{
#ifdef LOG_TO_FILE
	#define prnt0(a)		fprintf(f, a)
	#define prnt1(a, b)		fprintf(f, a, b)
#else /* LOG_TO_FILE */
	#define prnt0(a)		printf(a)
	#define prnt1(a, b)		printf(a, b)
#endif /* LOG_TO_FILE */

	prnt1("    [ GL ID    ][ %u ]\n", (u32) id);
}

#endif /* DEBUG */

/*---------------------------------------------------------------------------*/

