/*
 * Shader.
 * Inherits from Resource.
 *
 * A shader is either a vertex shader or a fragment shader. It is used as a
 * part of a whole shader program that is comprised of both a vertex shader
 * and a fragment shader.
 */


#ifndef __RC_SHADER_H__
#define __RC_SHADER_H__

/*----------------------------------------------------[ Shader settings ]----*/

typedef enum {

	SHADER_TYPE_VERTEX = 0,
	SHADER_TYPE_FRAGMENT,

	SHADER_TYPE_LAST

} ShaderType_t;

/*---------------------------------------------------------------------------*/

char *createPrefixString(
		char				**defs,
		u32					n_defs
	);


class Shader : public Resource {
	friend class SceneGraph;
	friend class ShaderProgram;

/*----------------------------------------------------------[ Functions ]----*/

	protected:
		Shader(
				const char			*new_key,
				char				*prefix,
				const char			*filename,
				ShaderType_t		type,
				u32					purge_level
			);

		virtual ~Shader();

	public:
		Resource *clone(
				const char				*clone_name,

				u32					purge_level
			);

#ifdef DEBUG
	protected:
		void print(
				FILE	*f
			);
#endif /* DEBUG */

/*------------------------------------------------------------[ Members ]----*/

	private:
		u32				id;

};

#endif /* __RC_SHADER_H__ */

