/*
 * ShaderProgram.
 * Inherits from Resource.
 */

#ifndef __RC_SHADERPROGRAM_H__
#define __RC_SHADERPROGRAM_H__

class ShaderProgram : public Resource {
	friend class SceneGraph;
	friend class Text;
	friend class Geometry;
	friend class Material;
	friend class TriangleMesh;
	friend class MatrixTransformation;
	friend class Sprite;

/*----------------------------------------------------------[ Functions ]----*/

	protected:
		ShaderProgram(
				const char			*nidentifier,
				Shader				*vertex_shader,
				Shader				*fragment_shader,
				u32					purge_level
			);

		~ShaderProgram();

	public:

		bool setAttribute(
				const char			*name,
				const f32			*values,
				u32					dimension
			);

		bool setAttributePointer(
				const char			*name,
				u32					size,
				bool				normalized,
				u32					stride,
				const void			*ptr
			);

		bool setUniform(
				const char			*name,
				const GLint			*values,
				u32					dimension
			);

		bool setUniform(
				const char			*name,
				const f32			*values,
				u32					dimension
			);

		bool setUniformVector(
				const char			*name,
				const i32			*values,
				u32					count,
				u32					dimension
			);

		bool setUniformVector(
				const char			*name,
				const f32			*values,
				u32					count,
				u32					dimension
			);

		bool setUniformMatrix(
				const char			*name,
				const f32			*values,
				u32					count,
				u32					dimension
			);

		bool setTextureSampler(
				const char			*name,
				Texture				*tex,
				u32					slot
			);

		bool setCubeMapSampler(
				const char			*name,
				CubeMap				*cubeMap,
				u32					slot
			);

		GLuint getID();

		Resource *clone(
				const char				*clone_name,

				u32					purge_level
			);

	protected:

		void activate();

		GLint getAttributeLocation(
				const char			*name
			);

		GLint getUniformLocation(
				const char			*name
			);

#ifdef DEBUG
	protected:
		void print(
				FILE	*f
			);
#endif /* DEBUG */

/*------------------------------------------------------------[ Members ]----*/

	private:
		Shader	*vertexShader;
		Shader	*fragmentShader;

		u32		matrixFlags;

		u32		id;

};

#endif /* __RC_SHADERPROGRAM_H__ */

