#include "RenderChimp.h"
#include "SceneGraph.h"
#include "RenderState.h"
#include "State.h"

/*---------------------------------------------------------------------------*/

State::State(
		const char	*nname
	) : Node(nname, NODE_STATE)
{
}

/*---------------------------------------------------------------------------*/

State::~State()
{

}

/*---------------------------------------------------------------------------*/

void State::draw(
		FrameInfo_t			*di
	)
{
	if (isActive) {
		StateSettings_t ts = *RenderState::getCurrentState();
		Node *c = firstChild;

		/* Update state for children */
		if (isVisible)
			RenderState::setState(&state.settings, state.settings.changes, false);

		/* Draw children */
		while (c) {
			c->draw(di);

			c = c->next;
		}

		/* Restore state from parent */
		if (isVisible)
			RenderState::setState(&ts, 0xFF, false);
	}
}

/*---------------------------------------------------------------------------*/

void State::enableDepth(
		StateCompare_t	depth_test
	)
{
	state.enableDepth(depth_test);
}

/*---------------------------------------------------------------------------*/

void State::disableDepth()
{
	state.disableDepth();
}

/*---------------------------------------------------------------------------*/

void State::enableStencil(
		StateCompare_t	stencil_test,
		i32				ref,
		u32				mask
	)
{
	state.enableStencil(stencil_test, ref, mask);
}

/*---------------------------------------------------------------------------*/

void State::disableStencil()
{
	state.disableStencil();
}

/*---------------------------------------------------------------------------*/

void State::enableBlend(
		StateSrcBlend_t	src_blend,
		StateDstBlend_t	dst_blend
	)
{
	state.enableBlend(src_blend, dst_blend);
}

/*---------------------------------------------------------------------------*/

void State::disableBlend()
{
	state.disableBlend();
}

/*---------------------------------------------------------------------------*/

void State::enableCulling(
		StateCulling_t	cull_strategy
	)
{
	state.enableCulling(cull_strategy);
}

/*---------------------------------------------------------------------------*/

void State::disableCulling()
{
	state.disableCulling();
}

/*---------------------------------------------------------------------------*/

void State::enableScissor(
		i32				x,
		i32				y,
		u32				width,
		u32				height
	)
{
	state.enableScissor(x, y, width, height);
}

/*---------------------------------------------------------------------------*/

void State::disableScissor()
{
	state.disableScissor();
}

/*---------------------------------------------------------------------------*/

Node *State::duplicateSelf()
{
	State *c = sceneGraph.createState(name);

	c->state = state;

	return c;
}

/*---------------------------------------------------------------------------*/

