/*
 * A bitmap font text renderer.
 *
 * Warning: Here be ugly code!
 */

/*
 * Special text styles can be achieved by using the escape char '~'
 * followed by a command in the input string:
 *
 *  cRGBA - Base color. RGBA are numbers in [0, 9]
 *  b - Bold
 *  i - Italic
 *  r - Reset all flags
 */

#ifndef __RC_TEXT_H__
#define __RC_TEXT_H__

#define MAX_PRINT_JOBS			32

/* Glyph description */
typedef struct {
	char c;				/* Glyph character */

	u32 flags;			/* Text style flags */

	rgbaf color;		/* Base color */

	i32 x;				/* x position in pixels */
	i32 y;				/* y position in pixels */
} Glyph_t;

/* Glyph vertex array */
/* struct {
 *		offset_x, y,
 *		glyph x, y,
 * }
 */
#define GLYPH_QUEUE_STRUCT_SIZE			4


class Text {
	friend class SceneGraph;

	public:
		Text(
				const char		*source_texture_filename,

				i32			line_spacing	/* Extra pixel space between lines */
			);

		~Text();
	
		/* Print all enqueued strings to screen */
		void printAll();

		/* Enqueue a string to be printed to screen */
		void print(
				const char	*string,	/* Print string */

				f32			from_x,		/* Location in screen coordinates [-1, 1] */
				f32			from_y,

				i32			row_width,	/* Max row width in pixels */
				i32			max_lines	/* Max number of lines of text */
			);

	private:
		/* Draws a glyph on screen */
		void drawGlyphs(
				rgbaf			color
			);

		void enqueueGlyph(
				GLfloat			x,
				GLfloat			y,
				Glyph_t			*g
			);

		void printOne(
				const char		*string,
				f32				from_x,
				f32				from_y,
				i32				row_width,
				i32				max_lines
			);

		/* Print a word or part of a long word */
		i32 printWord(
				Glyph_t *glyph,		/* Glyphs in the word */
				u32 length,			/* Number of glyphs */
				i32 x,				/* Pixel coords of beginning of word */
				i32 y,
				i32 word_width,		/* Width of word in pixels */
				i32 row_width,		/* Maximum allowed row width in pixels */
				i32 max_lines,		/* Maximum allowed lines */
				i32 *advance,		/* The cursor position */
				i32 *offset_y,		/* y offset with regard to line */
				i32	*current_line
			);

		/* Called to rescale the values to keep pixel size */
		void resize(
				f32 res_x,
				f32	res_y
			);

	private:
		GLuint			tex;						/* GL texture id */
		u32				width, height;				/* Size of texture (pixels) */
		u32				glyph_height, glyph_width;	/* Glyph size in texture (pixels) */
		u8				advance_base[256];			/* Advance of each char */
		GLfloat			glyph_tex_w, glyph_tex_h;	/* Glyph size in texture [0, 1] */
		i32				lineSpacing;				/* Line spacing in pixels */
		
		f32				resX, resY;					/* Device screen resolution */

		ShaderProgram	*sprog;						/* Text rendering shader program */
		
		char			*print_queue[MAX_PRINT_JOBS];
		f32				print_queue_x[MAX_PRINT_JOBS];
		f32				print_queue_y[MAX_PRINT_JOBS];
		i32				print_queue_rwdt[MAX_PRINT_JOBS];
		i32				print_queue_maxl[MAX_PRINT_JOBS];
		u32				print_queue_len[MAX_PRINT_JOBS];
		u32				n_print_jobs;

		rgbaf			lastColor;

		/* Glyph vertex array */
		/* struct {
		 *		offset_x, y,
		 *		glyph x, y,
		 *		R, G, B, A
		 *		(bold, italic)
		 * }
		 */
		u32				glyphQueueSize;
		u32				glyphQueueMax;
		f32				*glyphQueue;

};



#endif /* __RC_TEXT_H__ */


