/*
 * Defines a TriangleMesh node.
 * Inherits from Geometry.
 */

#ifndef __RC_TRIANGLEMESH_H__
#define __RC_TRIANGLEMESH_H__

class TriangleMesh : public Geometry {
	friend class SceneGraph;

/*----------------------------------------------------------[ Functions ]----*/

	protected:
		TriangleMesh(
				const char					*nname
			);

		TriangleMesh(
				const char					*nname,
				const char					*filename,

				bool					reload_resources,
				u32						purge_level
			);

		TriangleMesh(
				const char					*nname,

				VertexArray				*vertex_array,
				IndexArray				*index_array,
				Material				*nmaterial,

				bool					instantiate_resources
			);

		TriangleMesh(
				const char					*nname,

				/* These may be 0 */
				vec3f					*position_array,
				vec2f					*texcoord_array,
				vec3f					*normal_array,
				vec4f					*color_array,

				u32						n_vertices,

				u16						*index_array,
				u32						n_indices,
				IndexArrayLayout_t		index_layout,

				u32						purge_level
			);

		~TriangleMesh();

	public:

		void setVertexArray(
				VertexArray				*vertex_array,

				bool					instantiate_resource
			);

		void setVertexArray(
				vec3f					*position_array,
				vec2f					*texcoord_array,
				vec3f					*normal_array,
				vec4f					*color_array,
				u32						n_vertices,

				u32						purge_level
			);

		VertexArray *getVertexArray();

		void setIndexArray(
				IndexArray				*index_array,

				bool					instantiate_resource
			);

		void setIndexArray(
				u16						*index_array,
				u32						n_indices,

				IndexArrayLayout_t		layout,

				u32						purge_level
			);

		IndexArray *getIndexArray();

		void setMaterial(
				Material				*nmaterial,

				bool					instantiate_resource
			);

		Material *getMaterial();

	private:
		void prepareDrawing(
				FrameInfo_t				*di
			);

		void drawSelf(
				FrameInfo_t				*di
			);

		Node *duplicateSelf();

/*------------------------------------------------------------[ Members ]----*/

	private:
		VertexArray					*vertexArray;
		IndexArray					*indexArray;
		Material					*material;

};


#endif /* __RC_TRIANGLEMESH_H__ */

