
#include "../RenderChimp/RenderChimp.h"

#ifdef fopen
	#undef fopen
#endif

static NSString* iPhonePath(const char* filename) {
	int len = strlen(filename)-1;
	
	while (len >= 0 && filename[len] != '/')
		len--;
	
	filename = &filename[len+1];
	
	NSString* filenameString = [NSString stringWithCString:filename length:strlen(filename)];
	NSString* path = [[NSBundle mainBundle] pathForResource: filenameString ofType:nil];
	
	return path;
}

FILE* iphone_fopen(const char* filename, const char* mode) {
	NSString* path = iPhonePath(filename);
	return fopen([path cStringUsingEncoding:1], mode);
}

void apeGetTexture(const char* filename, u8** pixels, u32* outWidth, u32* outHeight, u32* bytes_per_pixel) {
	NSString* path = iPhonePath(filename);

	NSData* texData = [[NSData alloc] initWithContentsOfFile:path];
    UIImage* image = [[UIImage alloc] initWithData:texData];

    if (image == nil) {
		*pixels = 0;
		*outWidth = 0;
		*outHeight = 0;
		*bytes_per_pixel = 0;
	}

    GLuint width = CGImageGetWidth(image.CGImage);
    GLuint height = CGImageGetHeight(image.CGImage);
	GLint bpp = CGImageGetBitsPerPixel(image.CGImage)/8;

    void* imageData;
	CGColorSpaceRef colorSpace; 
	CGContextRef context;
	
	if (bpp == 3) {
		imageData = malloc(height*width*4);
		colorSpace = CGColorSpaceCreateDeviceRGB();
		context = CGBitmapContextCreate(imageData, width, height, 8, 4 * width, colorSpace, kCGImageAlphaNoneSkipLast | kCGBitmapByteOrder32Big);
		bpp = 4;
	} else if (bpp == 4) {
		imageData = malloc(height*width*4);
		colorSpace = CGColorSpaceCreateDeviceRGB();
		context = CGBitmapContextCreate(imageData, width, height, 8, 4 * width, colorSpace, kCGImageAlphaPremultipliedLast | kCGBitmapByteOrder32Big);
		bpp = 4;
	} else if (bpp == 1) {
		imageData = malloc(height*width*bpp);
		colorSpace = CGImageGetColorSpace(image.CGImage);
        context = CGBitmapContextCreate(imageData, width, height, 8, bpp*width, colorSpace, CGImageGetBitmapInfo(image.CGImage));
	} else {
		exit(1);
	}
	
	
    CGColorSpaceRelease(colorSpace);

    CGContextClearRect(context, CGRectMake(0, 0, width, height));
    CGContextDrawImage(context, CGRectMake(0, 0, width, height), image.CGImage);

    CGContextRelease(context);
	[image release];
	[texData release];

	*pixels = (u8*)imageData;
	*outWidth = width;
	*outHeight = height;
	*bytes_per_pixel = bpp;
}

void apeDestroyTexture(unsigned char* pixels) {
	free(pixels);
}

void apeSetViewport(
					i32					x,
					i32					y,
					u32					width,
					u32					height
					)
{
	glViewport(x, y, width, height);
}

void apeGetDisplaySize(
					   u32					*width,
					   u32					*height
					   )
{
	// HACK - assume potrait, should query iphone orientation
	*width = 320; 
	*height = 480;
}

