#ifdef GL_FRAGMENT_PRECISION_HIGH
   precision highp float;
#else
   precision mediump float;
#endif

#ifdef DIRECTIONAL
	uniform vec3 LightVector;
#else /* POINT */
	uniform vec3 LightPosition;
#endif

uniform vec3 LightColor;

uniform	vec3 AmbientColor;
uniform	vec3 DiffuseColor;
uniform	float Emissive;
uniform	vec3 SpecularColor;
uniform	float Shininess;
uniform	float SpecularLevel;

uniform sampler2D DiffuseSampler;

varying	vec2 texcoord;
varying	vec3 worldPosition;
varying	vec3 worldNormal;
varying	vec3 viewVector;

varying vec3 diffuse_color;


vec4 specularPhong(
		float specular_level
	)
{
	vec3 specular = vec3(0.0);

	if (specular_level > 0.0) {
		vec3 normWorldNormal = normalize(worldNormal);

#ifdef DIRECTIONAL
		vec3 lightVector = LightVector;
#else /* POINT */
		vec3 lightVector = normalize(LightPosition - worldPosition);
#endif

		vec3 reflected = reflect(lightVector, normWorldNormal);
		float s = max(0.0, dot(normalize(reflected), viewVector));
		specular = LightColor * pow(s, Shininess) * specular_level * SpecularColor;

	}

	return vec4(specular, 0.0);
}

void main() {

	vec4 color = vec4(diffuse_color, 1.0);
	
	    // Diffuse texture
	color *= texture2D(DiffuseSampler, texcoord);

		// Specular
	color += specularPhong(SpecularLevel);

		// Write final color to color buffer
	gl_FragColor = color;

}
