/*
 * .pwn Format definition
 *
 * This file functions as the actual definition of the .pwn format. Any pwn-
 * writer or reader has to comply with the format specified here.
 *
 */

#ifndef __PWN_H__
#define __PWN_H__

#define PWN_ID_MAX_LENGTH					64
#define PWN_MESH_MAX_MATERIALS				256
#define PWN_MESH_MAX_MESHES					256
#define PWN_MESH_MAX_ATTRIBUTES				16
#define PWN_MESH_MAX_ATTRIB_NAME_LENGTH		32




/*
 * The .pwn format has the following blocks:
 *
 *  - Header
 *  - Materials
 *  - Mesh data
 *  - Node hierarchy
 *
 */

/* Header block */

		/* Pwn identifier (uint) */
#define PWN_ID							"pwnd"

		/* Format version (uint) (Latest is 1)*/
#define PWN_VERSION						1

	/* n Materials (uint) */
#define PWN_N_MATERIALS_OFFSET			8

	/* n Meshes (uint) */
#define PWN_N_MESHES_OFFSET				12


/* Material block */

		/* Each material starts with... (uint) */
#define PWN_MATERIAL_PREFIX				"matl"

		/* These can come in any order */

		/* -- Required attributes (uint) */

#define PWN_MATERIAL_ID					"idnt"
	/* length of id (uint) */
	/* id string (char[])*/

		/* -- Optional attributes */

#define PWN_MATERIAL_AMB_COLOR_PREFIX	"ambi"
	/* R, G, B (3 floats) */

#define PWN_MATERIAL_DIF_COLOR_PREFIX	"diff"
	/* R, G, B (3 floats) */

#define PWN_MATERIAL_SPC_COLOR_PREFIX	"spec"
	/* R, G, B (3 floats) */

#define PWN_MATERIAL_SHI_COLOR_PREFIX	"shin"
	/* value (float) */

#define PWN_MATERIAL_EMI_COLOR_PREFIX	"emis"
	/* value (float) */

#define PWN_MATERIAL_SPL_COLOR_PREFIX	"spcl"
	/* value (float) */

/* Mesh block */

		/* Each mesh starts with... (uint) */
#define PWN_MESH_PREFIX					"mesh"

		/* -- Required attributes */

#define PWN_MESH_ID						"idnt"
	/* length of id (uint) */
	/* id string (char[]) */

#define PWN_MESH_MATERIAL				"mmat"
	/* material index (uint) */

#define PWN_MESH_ATTRIBUTES				"attr"
	/* n attributes (uint) */

		/* 1..n vertex attribute names. Listed in order. */
	/* length of material id (uint) */
	/* id string (char[])*/
	/* offset in the vertex array (uint) */
	/* n of elements (uint) */

#define PWN_MESH_INDEX_ARRAY			"idxa"
	/* n indices (uint) */
	/* type size (uint) (4 if n indices > 65535, otherwise 2) */
	/* indices (ushort[] or uint[])*/

#define PWN_MESH_VERTEX_ARRAY			"vtxa"
	/* n vertex attributes (uint) */
	/* stride (uint) */
	/* attributes (float[] (n * stride of them0))*/


/* Node hierarchy block */

		/* Node hierarchy is listed in a depth-first manner */

		/* The node starts with... (uint) */
#define PWN_NODE_PREFIX					"node"

		/* -- Optional attributes */

#define PWN_NODE_ROTATE					"rota"
	/* angle, x, y, z (4 floats) */

#define PWN_NODE_TRANSLATE				"tran"
	/* x, y, z (3 floats) */

#define PWN_NODE_SCALE					"scal"
	/* x, y, z (3 floats) */


		/* -- Required attributes */

		/* next follows a node type tag (select one) */

	/* node type (uint) */ /* (Currently, only meshes are allowed) */
#define PWN_NODE_TYPE_MESH				"nmsh"
//#define PWN_NODE_TYPE_LIGHT				"nlgt"		/* TODO */
//#define PWN_NODE_TYPE_CAMERA				"ncam"		/* TODO */

	/* index to corresponding mesh (or other) node (uint) */
	/* n children (uint) */
	/* children list (list of new PWN_NODEs) */




#define PWN_END_FILE					"xxxx"

#endif /* __PWN_H__ */

